-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function Swarm:StartActivity()
   	print("START! -- Swarm:StartActivity()!");

	-- self.SweepZone = SceneMan.Scene:GetArea("SweepZone"); -- Zone to clear out of enemies
    self.FightPhase = { NOFIGHT = 0 , OUTPOSTTAKEN = 1 };
    self.CurrentFightPhase = self.FightPhase.NOFIGHT;


	self.StartLZ = SceneMan.Scene:GetArea("StartLZ");
	self.OutpostLZ = SceneMan.Scene:GetArea("OutpostLZ");
	
	self.OutpostBridge = SceneMan.Scene:GetArea("OutpostBridge");
	self.BaseBridge = SceneMan.Scene:GetArea("BaseBridge");

	-- To optimize zone scanning we define three master zones
	-- Scanner first scans if actor is in master zone if not
	-- it skips 5 to 10 additional checks
	self.OutpostZone = SceneMan.Scene:GetArea("OutpostZone"); -- Includes outpost firing points 1 to 5
	self.ScanZone1 = SceneMan.Scene:GetArea("ScanZone1"); -- Includes all firing point from 6 to 15
	self.ScanZone2 = SceneMan.Scene:GetArea("ScanZone2"); -- Includes all firing point from 16 to 23
	self.BaseVatsChamber = SceneMan.Scene:GetArea("BaseVatsChamber");
	
	


   	-- Set up teams
   	-- Team 2 is always CPU
   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;

	self.braindead = {};
	self.AssignedBrains = {};
	-- Notice that CPU uses MOIDLimit + 25 to control it's population and guarantee brain hall\vats reinforcements
	-- when MOID limit is hit
	self.MOIDLimit = 190;
    
    self.GenericTimer = Timer();
	self.TeleportationTimer = Timer();
	self.CloningVatsDesyncTimer = Timer();
	
	self.DummyDropshipSpawnTimer = Timer();
	self.DummySpawnTimer = Timer();
	self.DummyGoldTimer = Timer();

	self.LightWList = { "Blaster", "Nailgun" , "Rail Pistol" , "Scouting Rifle" , "Repeater" , "Impulse Cannon"};
	self.HeavyWList = { "Blaster" , "Scouting Rifle" , "Repeater" , "Impulse Cannon" , "Annihiliator", "Destroyer Cannon", "Nailer Cannon" };
	
	-- Zones held by coalition troops
	-- Vars for meaningfull indexing of outpost positions
	self.OutpostRoof = 1; 
	self.OutpostUp = 2;
	self.OutpostMid = 3;
	self.OutpostLow = 4;
	self.OutpostFloor = 5;

	self.BaseFrontUpRoof = 6;
	self.BaseFrontUpUp = 7;
	self.BaseFrontUpMid = 8;
	self.BaseFrontUpLow = 9;

	self.BaseFrontLowRoof = 10;
	self.BaseFrontLowUp = 11;
	self.BaseFrontLowMid = 12;
	self.BaseFrontLowLow = 13;

	self.BaseTowerUp = 14;
	self.BaseTowerLow = 15;

	self.BaseRoof = 16;

	self.BaseChamberUp = 17;
	self.BaseChamberMid = 18;
	self.BaseChamberLow = 19;

	self.BaseDisplayHall = 20;

	self.BaseBrainHallRight = 21;
	self.BaseBrainHallMid = 22;
	self.BaseBrainHallLeft = 23;
	
	--[[
	-- Zones descriptions for debugging
	self.ZonesDesc = {};
	self.ZonesDesc[1] = "OutpostRoof = 1"; 
	self.ZonesDesc[2] = "OutpostUp = 2";
	self.ZonesDesc[3] = "OutpostMid = 3";
	self.ZonesDesc[4] = "OutpostLow = 4";
	self.ZonesDesc[5] = "OutpostFloor = 5";

	self.ZonesDesc[6] = "BaseFrontUpRoof = 6";
	self.ZonesDesc[7] = "BaseFrontUpUp = 7";
	self.ZonesDesc[8] = "BaseFrontUpMid = 8";
	self.ZonesDesc[9] = "BaseFrontUpLow = 9";

	self.ZonesDesc[10] = "BaseFrontLowRoof = 10";
	self.ZonesDesc[11] = "BaseFrontLowUp = 11";
	self.ZonesDesc[12] = "BaseFrontLowMid = 12";
	self.ZonesDesc[13] = "BaseFrontLowLow = 13";

	self.ZonesDesc[14] = "BaseTowerUp = 14";
	self.ZonesDesc[15] = "BaseTowerLow = 15";

	self.ZonesDesc[16] = "BaseRoof = 16";

	self.ZonesDesc[17] = "BaseChamberUp = 17";
	self.ZonesDesc[18] = "BaseChamberMid = 18";
	self.ZonesDesc[19] = "BaseChamberLow = 19";

	self.ZonesDesc[10] = "BaseDisplayHall = 20";

	self.ZonesDesc[21] = "BaseBrainHallRight = 21";
	self.ZonesDesc[22] = "BaseBrainHallMid = 22";
	self.ZonesDesc[23] = "BaseBrainHallLeft = 23";
	]]--
	
	-- Zones defining firing point checked for troops precence
	self.Zones = {}; 
	-- Coalition firing positions at outpost	
	-- Can be indexed with outpost positions variables
	self.Zones[1] = SceneMan.Scene:GetArea("Outpost_Roof");
	self.Zones[2] = SceneMan.Scene:GetArea("Outpost_Up");
	self.Zones[3] = SceneMan.Scene:GetArea("Outpost_Mid");
	self.Zones[4] = SceneMan.Scene:GetArea("Outpost_Low");
	self.Zones[5] = SceneMan.Scene:GetArea("Outpost_Floor");

	self.Zones[6] = SceneMan.Scene:GetArea("BaseFrontUpRoof");
	self.Zones[7] = SceneMan.Scene:GetArea("BaseFrontUpUp");
	self.Zones[8] = SceneMan.Scene:GetArea("BaseFrontUpMid");
	self.Zones[9] = SceneMan.Scene:GetArea("BaseFrontUpLow");

	self.Zones[10] = SceneMan.Scene:GetArea("BaseFrontLowRoof");
	self.Zones[11] = SceneMan.Scene:GetArea("BaseFrontLowUp");
	self.Zones[12] = SceneMan.Scene:GetArea("BaseFrontLowMid");
	self.Zones[13] = SceneMan.Scene:GetArea("BaseFrontLowLow");

	self.Zones[14] = SceneMan.Scene:GetArea("BaseTowerUp");
	self.Zones[15] = SceneMan.Scene:GetArea("BaseTowerLow");

	self.Zones[16] = SceneMan.Scene:GetArea("BaseRoof");

	self.Zones[17] = SceneMan.Scene:GetArea("BaseChamberUp");
	self.Zones[18] = SceneMan.Scene:GetArea("BaseChamberMid");
	self.Zones[19] = SceneMan.Scene:GetArea("BaseChamberLow");
	
	self.Zones[20] = SceneMan.Scene:GetArea("BaseDisplayHall");

	self.Zones[21] = SceneMan.Scene:GetArea("BaseBrainHallRight");
	self.Zones[22] = SceneMan.Scene:GetArea("BaseBrainHallMid");
	self.Zones[23] = SceneMan.Scene:GetArea("BaseBrainHallLeft");

	
	-- Destination points. Troops GOTO to these coordinates when they are teleported
	self.ZoneDestinations = {}; 
	-- Coalition firing positions for actors at outpost
	-- Can be indexed with outpost positions variables
	self.ZoneDestinations[1] = Vector(2727 , 700); 
	self.ZoneDestinations[2] = Vector(2652 , 796); 
	self.ZoneDestinations[3] = Vector(2652 , 892);
	self.ZoneDestinations[4] = Vector(2652 , 988);
	self.ZoneDestinations[5] = Vector(2680 , 1088);

	self.ZoneDestinations[6] = Vector(1245 , 461);
	self.ZoneDestinations[7] = Vector(1238 , 533);
	self.ZoneDestinations[8] = Vector(1188 , 629);
	self.ZoneDestinations[9] = Vector(1145 , 724);

	self.ZoneDestinations[10] = Vector(1434 , 748);
	self.ZoneDestinations[11] = Vector(1427 , 821);
	self.ZoneDestinations[12] = Vector(1381 , 916);
	self.ZoneDestinations[13] = Vector(1335 , 1012);

	self.ZoneDestinations[14] = Vector(1001 , 53);
	self.ZoneDestinations[15] = Vector(1049 , 244);
	
	self.ZoneDestinations[16] = Vector(518 , 460);

	self.ZoneDestinations[17] = Vector(587 , 605);
	self.ZoneDestinations[18] = Vector(585 , 724);
	self.ZoneDestinations[19] = Vector(693 , 832);

	self.ZoneDestinations[20] = Vector(591 , 916);

	self.ZoneDestinations[21] = Vector(1051 , 1085);
	self.ZoneDestinations[22] = Vector(709 , 1049);
	self.ZoneDestinations[23] = Vector(465 , 1047);

	
	-- Assigned weapons. Troops obtain these weapons when they are teleported
	self.ZoneWeapons = {}; 
	self.ZoneWeapons[1] = {"Missile Launcher"}; -- Weapons for coalition soldiers at firing positions
	self.ZoneWeapons[2] = {"Assault Rifle" , "Sniper Rifle"}; -- Can be indexed with outpost positions variables
	self.ZoneWeapons[3] = {"Assault Rifle" , "Sniper Rifle" , "Shotgun"};
	self.ZoneWeapons[4] = {"Assault Rifle" , "Shotgun" , "Flamer"};
	self.ZoneWeapons[5] = {"Shotgun" , "Flamer"};

	self.ZoneWeapons[6] = {"Missile Launcher", "Heavy Sniper Rifle"};
	self.ZoneWeapons[7] = {"Sniper Rifle", "Assault Rifle" , "Heavy Sniper Rifle"};
	self.ZoneWeapons[8] = {"Sniper Rifle", "Compact Assault Rifle" , "Shotgun"};
	self.ZoneWeapons[9] = {"Shotgun" , "Flamer" , "Grenade Launcher"};

	self.ZoneWeapons[10] = {"Sniper Rifle", "Assault Rifle" , "Auto Shotgun"};
	self.ZoneWeapons[11] = {"Sniper Rifle", "Assault Rifle" , "Heavy Sniper Rifle"};
	self.ZoneWeapons[12] = {"Sniper Rifle", "Compact Assault Rifle" , "Shotgun"};
	self.ZoneWeapons[13] = {"Auto Shotgun" , "Auto Cannon" , "Flamer" , "Auto Cannon"};

	self.ZoneWeapons[14] = {"Missile Launcher" , "Heavy Sniper Rifle"};
	self.ZoneWeapons[15] = {"Missile Launcher" , "Heavy Sniper Rifle"};

	self.ZoneWeapons[16] = {"Auto Shotgun", "Missile Launcher" , "Heavy Sniper Rifle" , "Auto Cannon"};

	self.ZoneWeapons[17] = {"Assault Rifle" , "Auto Shotgun"};
	self.ZoneWeapons[18] = {"Assault Rifle" , "Shotgun" , "Auto Cannon"};
	self.ZoneWeapons[19] = {"Assault Rifle" , "Auto Shotgun"};
	
	self.ZoneWeapons[20] = {"Assault Rifle" , "Auto Shotgun"};

	self.ZoneWeapons[21] = {"Assault Rifle" , "Auto Shotgun" , "Flamer"};
	self.ZoneWeapons[22] = {"Assault Rifle" , "Auto Shotgun" , "Flamer"};
	self.ZoneWeapons[23] = {"Auto Shotgun"};

	-- Empty firing points flags
	self.IsZoneEmpty = {}; 
	-- Time when troops should be sent to point next time
	-- We need to give troops some time to reach destination area before sending there again
	self.NextTroopsSendTime = {}; 
	
	-- Reset flags in counters
	for i = 1, 23 do
		self.IsZoneEmpty[i] = true;
		self.NextTroopsSendTime[i] = 0;
	end
	
	-- Manually set last send time for those point where troops are already preinstalled
	-- as addedactor are not indexed by zone checker
	self.NextTroopsSendTime[self.OutpostRoof] = 20000;
	self.NextTroopsSendTime[self.OutpostMid] = 20000;
	self.NextTroopsSendTime[self.OutpostFloor] = 20000;
	
	self.NextTroopsSendTime[self.BaseFrontUpLow] = 20000;
	self.NextTroopsSendTime[self.BaseFrontLowLow] = 20000;
	self.NextTroopsSendTime[self.BaseTowerUp] = 20000;

	self.NextTroopsSendTime[self.BaseChamberUp] = 20000;
	self.NextTroopsSendTime[self.BaseChamberMid] = 20000;
	self.NextTroopsSendTime[self.BaseChamberLow] = 20000;	

	-- Brain hall areas taken by enemy flags
	self.IsBrainHallTaken = {}; 
	for i = 1, 3 do
		self.IsBrainHallTaken[i] = false;
	end

	-- Vats zone scanned for enemy precence
	self.BaseVatsZones = {};
	self.BaseVatsZones[1] = SceneMan.Scene:GetArea("BaseVatsChamber");
	self.BaseVatsZones[2] = SceneMan.Scene:GetArea("BaseVatsEntranceUp");
	self.BaseVatsZones[3] = SceneMan.Scene:GetArea("BaseVatsEntranceLow");

	-- Vats areas taken by enemy flags
	self.BaseVatsZonesTaken = {};
	self.BaseVatsZonesTaken[1] = false;
	self.BaseVatsZonesTaken[2] = false;
	self.BaseVatsZonesTaken[3] = false;

	-- Flag set to true if any of the brain zone contains enemies
	self.BrainThreatened = false;
	-- Flag set to true if any of the vats zone contains enemies
	self.VatsThreatened = false;
	
	-- A-Teleporters list with source coordinates used in global teleportation loop
	self.TeleportersA = {};
	
	self.TeleportersA[1] = Box(Vector(108 - 18, 648 - 18) , 36 , 36);
	self.TeleportersA[2] = Box(Vector(108 - 18, 816 - 18) , 36 , 36);
	self.TeleportersA[3] = Box(Vector(300 - 18, 648 - 18) , 36 , 36);
	self.TeleportersA[4] = Box(Vector(300 - 18, 816 - 18) , 36 , 36);
	self.TeleportersA[5] = Box(Vector(492 - 18, 648 - 18) , 36 , 36);
	self.TeleportersA[6] = Box(Vector(492 - 18, 816 - 18) , 36 , 36);
	
	-- Vars for meaningfull indexing of B-teleporters
	self.TeleporterBOutpostRoof = 1;
	self.TeleporterBOutpostUp = 2;
	self.TeleporterBOutpostLow = 3;

	self.TeleporterBBaseFrontUp = 4;
	self.TeleporterBBaseFrontLow = 5;

	self.TeleporterBBaseRoofLeft = 6;
	self.TeleporterBBaseRoofRight = 7;

	self.TeleporterBBaseTowerUp = 8;
	self.TeleporterBBaseTowerLow = 9;
	
	self.TeleporterBBaseInner = 10;
	self.TeleporterBBaseHall = 11;
	
	self.TeleporterBBaseBrainHallLeft = 12;
	self.TeleporterBBaseBrainHallMid = 13;
	self.TeleporterBBaseBrainHallRight = 14;

	--[[
	-- B-teleporters descriptions for debugging
	self.TeleportersBDesc = {}; 
	self.TeleportersBDesc[1] = "TeleporterBOutpostRoof = 1";
	self.TeleportersBDesc[2] = "TeleporterBOutpostUp = 2";
	self.TeleportersBDesc[3] = "TeleporterBOutpostLow = 3";

	self.TeleportersBDesc[4] = "TeleporterBBaseFrontUp = 4";
	self.TeleportersBDesc[5] = "TeleporterBBaseFrontLow = 5";

	self.TeleportersBDesc[6] = "TeleporterBBaseRoofLeft = 6";
	self.TeleportersBDesc[7] = "TeleporterBBaseRoofRight = 7";

	self.TeleportersBDesc[8] = "TeleporterBBaseTowerUp = 8";
	self.TeleportersBDesc[9] = "TeleporterBBaseTowerLow = 9";
	
	self.TeleportersBDesc[10] = "TeleporterBBaseInner = 10";
	self.TeleportersBDesc[11] = "TeleporterBBaseHall = 11";
	
	self.TeleportersBDesc[12] = "TeleporterBBaseBrainHallLeft = 12";
	self.TeleportersBDesc[13] = "TeleporterBBaseBrainHallMid = 13";
	self.TeleportersBDesc[14] = "TeleporterBBaseBrainHallRight = 14";
	]]--
	
	-- Add B-Teleporter receive delay to avoid crowds of troops
	-- teleported to the same B-Teleport
	self.NextTeleporterReceiveTime = {}; 
	for i = 1, 14 do
		self.NextTeleporterReceiveTime[i] = 0;
	end
	
	-- B-Teleporters list with destination coordinates
	self.TeleportersB = {}; 
	self.TeleportersB[1] = Vector(2460 , 552 - 2); -- self.TeleporterBOutpostRoof
	self.TeleportersB[2] = Vector(2292 , 792 - 2); -- self.TeleporterBOutpostUp
	self.TeleportersB[3] = Vector(2292 , 984 - 2); -- self.TeleporterBOutpostLow

	self.TeleportersB[4] = Vector(1020 , 528 - 2); -- self.TeleporterBBaseFrontUp
	self.TeleportersB[5] = Vector(1212 , 816 - 2); -- self.TeleporterBBaseFrontLow

	self.TeleportersB[6] = Vector(744 , 432 - 2); -- self.TeleporterBBaseRoofLeft
	self.TeleportersB[7] = Vector(1008 , 432 - 2); -- self.TeleporterBBaseRoofRight
	
	self.TeleportersB[8] = Vector(756 , 168 - 2); -- self.TeleporterBBaseTowerUp
	self.TeleportersB[9] = Vector(828 , 240 - 2); -- self.TeleporterBBaseTowerLow

	self.TeleportersB[10] = Vector(828 , 624 - 2); -- self.TeleporterBBaseInner
	self.TeleportersB[11] = Vector(540 , 912 - 2); -- self.TeleporterBBaseHall

	self.TeleportersB[12] = Vector(252 , 912 - 2); -- self.TeleporterBBaseBrainHallLeft
	self.TeleportersB[13] = Vector(576 , 984 - 2); -- self.TeleporterBBaseBrainHallMid
	self.TeleportersB[14] = Vector(852 , 984 - 2); -- self.TeleporterBBaseBrainHallRight

	-- Assign controlled points to responsible B-teleporters
	-- These are indexed by zone meaningfull indexing variables,
	self.ZoneToTeleport = {};

	-- Zone-to-Teleport assignment self.ZoneToTeleport[zone index] = destination b-teleport index
	self.ZoneToTeleport[self.OutpostRoof] = self.TeleporterBOutpostRoof;
	self.ZoneToTeleport[self.OutpostUp] = self.TeleporterBOutpostUp;
	self.ZoneToTeleport[self.OutpostMid] = self.TeleporterBOutpostUp;
	self.ZoneToTeleport[self.OutpostLow] = self.TeleporterBOutpostLow;
	self.ZoneToTeleport[self.OutpostFloor] = self.TeleporterBOutpostLow;
	
	self.ZoneToTeleport[self.BaseFrontUpRoof] = self.TeleporterBBaseRoofRight;
	self.ZoneToTeleport[self.BaseFrontUpUp] = self.TeleporterBBaseFrontUp;
	self.ZoneToTeleport[self.BaseFrontUpMid] = self.TeleporterBBaseFrontUp;
	self.ZoneToTeleport[self.BaseFrontUpLow] = self.TeleporterBBaseFrontUp;

	self.ZoneToTeleport[self.BaseFrontLowRoof] = self.TeleporterBBaseRoofRight;
	self.ZoneToTeleport[self.BaseFrontLowUp] = self.TeleporterBBaseFrontLow;
	self.ZoneToTeleport[self.BaseFrontLowMid] = self.TeleporterBBaseFrontLow;
	self.ZoneToTeleport[self.BaseFrontLowLow] = self.TeleporterBBaseFrontLow;

	self.ZoneToTeleport[self.BaseTowerUp] = self.TeleporterBBaseTowerUp;
	self.ZoneToTeleport[self.BaseTowerLow] = self.TeleporterBBaseTowerLow;

	self.ZoneToTeleport[self.BaseRoof] = self.TeleporterBBaseRoofLeft;

	self.ZoneToTeleport[self.BaseChamberUp] = self.TeleporterBBaseRoofLeft;
	self.ZoneToTeleport[self.BaseChamberMid] = self.TeleporterBBaseInner;
	self.ZoneToTeleport[self.BaseChamberLow] = self.TeleporterBBaseInner;

	self.ZoneToTeleport[self.BaseDisplayHall] = self.TeleporterBBaseHall;

	self.ZoneToTeleport[self.BaseBrainHallRight] = self.TeleporterBBaseBrainHallRight;
	self.ZoneToTeleport[self.BaseBrainHallMid] = self.TeleporterBBaseBrainHallMid;
	self.ZoneToTeleport[self.BaseBrainHallLeft] = self.TeleporterBBaseBrainHallLeft;
	
	-- Add cloning vats
	self.CloningEnabled = false;
	
	self.Generators = {};
	
   	for i = 1 , 6 do
		self.Generators[i] = CreateAEmitter("Clone Generator" , "Swarm.rte");
		self.Generators[i].Team = self.CPUTeam;
		self.Generators[i]:EnableEmission(false);
		MovableMan:AddParticle(self.Generators[i]);
	end

	self.Generators[1].Pos = Vector(108, 554);
	self.Generators[2].Pos = Vector(108, 722);

	self.Generators[3].Pos = Vector(300, 554);
	self.Generators[4].Pos = Vector(300, 722);

	self.Generators[5].Pos = Vector(492, 554);
	self.Generators[6].Pos = Vector(492, 722);

	self.PowerGenerators = {}
	
   	for i = 1 , 4 do
		self.PowerGenerators[i] = CreateMOSRotating("Power Generator" , "Swarm.rte");
		self.PowerGenerators[i].Team = self.CPUTeam;
		MovableMan:AddParticle(self.PowerGenerators[i]);
	end

	self.PowerGenerators[1].Pos = Vector(2280, 884);
	self.PowerGenerators[2].Pos = Vector(2280, 900);

	self.PowerGenerators[3].Pos = Vector(2304, 884);
	self.PowerGenerators[4].Pos = Vector(2304, 900);

	-- Decision making or usefull stats
	self.CoalitionCount = 0;
	self.DummyCount = 0;
	self.UnitCount = 0;
	
	-- Generic timing and money parameters
	self.PlayerGold = 0; -- Player starting credits

	self.DummyDropshipSpawnInterval = 16500;
	self.DummySpawnInterval = 11500;
	self.DummyGoldInterval = 120;
	self.DummyGoldAmount = 3;
	-- Flag to avoid constant triggering of winning routine
	self.CPUBrainDead = false;
	
	-- Amount of time to wait for clones to reach their firing point 
	-- before sending troops to the same point again
	self.FiringPointReachDelay = 15000;
	-- Amount of time to delay receiving teleport
	self.TeleporterReceiveDelay = 15000;
	
	-- Spawn CPU brain
	self.CPUBrain = CreateAHuman("Brain Robot" , "Base.rte");
	self.CPUBrain:AddInventoryItem(CreateHDFirearm("Auto Pistol" , "Coalition.rte"));
	self.CPUBrain:AddInventoryItem(CreateHDFirearm("Compact Assault Rifle" , "Coalition.rte"));
	self.CPUBrain.Team = self.CPUTeam;
	self.CPUBrain.Pos = Vector(130 , 1000);
	self.CPUBrain.AIMode = Actor.AIMODE_SENTRY;
	MovableMan:AddActor(self.CPUBrain);

	-- Spawn dropship with dreadnought
	local dropship = CreateACDropShip("Drop Ship");
	local crab = CreateACrab("Dreadnought");
	crab.Team = self.PlayerTeam;
	crab.AIMode = Actor.AIMODE_BRAINHUNT;
	dropship:AddInventoryItem(crab);
	dropship.Pos = Vector(3850 , 250);
	dropship.Team = self.PlayerTeam;
	dropship:SetControllerMode(Controller.CIM_AI, -1);
	MovableMan:AddActor(dropship);
    
   	for player = 0, self.PlayerCount - 1 do
    -- Check if we already have a brain assigned
		if not self:GetPlayerBrain(player) then
			-- Set up players
			self.braindead[player] = false;
			--local foundBrain = MovableMan:GetUnassignedBrain(self:GetTeamOfPlayer(player));
			-- Create a rocket with a brain bot inside instead
			local dropship = CreateACDropShip("Drop Ship");
			local brain = CreateAHuman("Dummy Walking Controller" , "Swarm.rte");
			brain:AddInventoryItem(CreateHDFirearm("Blaster" , "Dummy.rte"));
			brain:AddInventoryItem(CreateHDFirearm("Scouting Rifle" , "Dummy.rte"));
			brain.Team = self.PlayerTeam;
			brain.AIMode = Actor.AIMODE_SENTRY;

			dropship:AddInventoryItem(brain);
			dropship.Pos = Vector(4250 - 50 * (player + 1), 300);
			dropship.Team = self:GetTeamOfPlayer(player);
			dropship:SetControllerMode(Controller.CIM_AI, -1);
			-- Let the spawn into the world, passing ownership
			MovableMan:AddActor(dropship);
			foundBrain = dropship;
			self.AssignedBrains[player] = dropship;

			-- If we can't find an unassigned brain in the scene to give each player, then force to go into editing mode to place one
			-- Set the found brain to be the selected actor at start
			self:SetPlayerBrain(foundBrain, player);
			self:SetViewState(Activity.ACTORSELECT, player);
			self:SetActorSelectCursor(Vector(4250, 350), player);
			-- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
			self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player)
		end --if
	end --for

  
    -- Set up AI modes for the Actors that have been added to the scene by the scene definition
    for actor in MovableMan.AddedActors do
		if actor.Team == self.CPUTeam then
			actor.AIMode = Actor.AIMODE_SENTRY;
		end
		if actor.Team == self.PlayerTeam then
			actor.AIMode = Actor.AIMODE_BRAINHUNT;
		end
    end

    -- Set the team's funds.
    self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);

    -- Reset all timers
	self.GenericTimer:Reset();
	self.TeleportationTimer:Reset();
end

-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function Swarm:PauseActivity(pause)
    print("PAUSE! -- DummyAssault:PauseActivity()!");
end

-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function Swarm:EndActivity()
    print("END! -- DummyAssault:EndActivity()!");
end

-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function Swarm:UpdateActivity()
   	-- Clear all objective markers, they get re-added each frame
   	self:ClearObjectivePoints();
   	
   	self:DoBrainSelection();
	self:DoCloneBreedingControl();
	self:ActorSearchLoop();
	self:TeleportControlLoop(); -- TeleportControlLoop should follow ActorSearchLoop
	self:SendReinforcements();

	if self.CurrentFightPhase == self.FightPhase.NOFIGHT then
		self:PhaseNoFight();
	elseif self.CurrentFightPhase == self.FightPhase.OUTPOSTTAKEN then
		self:PhaseOutpostTaken();
	end
	
	self:CheckWinningConditions()	
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function Swarm:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		-- Iterate through all human players
		for player = 0, self.PlayerCount - 1 do
			-- The current player's team
			local team = self:GetTeamOfPlayer(player);
	
			-- Make sure the game is not already ending
			-- Check if any player's brain is dead
			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				self:SetPlayerBrain(nil, player);
				-- Try to find a new unasigned brain this player can use instead, or if his old brain entered a craft
				local newBrain = MovableMan:GetUnassignedBrain(team);
				-- Found new brain actor, assign it and keep on truckin'
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self.AssignedBrains[player] = newBrain;
					self:SwitchToActor(newBrain, player, team);
				else
					FrameMan:SetScreenText("Commander died!", player, 333, -1, false);
					self.braindead[player] = true;
					
					-- Now see if all brains of self player's team are dead, and if so, end the game
					--if not MovableMan:GetClosestBrainActor(team) then
						self.WinnerTeam = self:OtherTeam(team);
						ActivityMan:EndActivity();
					--end
				end
			else
				-- Update the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end
end
-----------------------------------------------------------------------------------------
-- Turn cloning vats on and off to prevent MOID limit hit
-----------------------------------------------------------------------------------------
function Swarm:DoCloneBreedingControl()
	local currentmoidlimit;
	
	if self.BrainThreatened or self.VatsThreatened then
		-- print ("MOID boost");
		currentmoidlimit = self.MOIDLimit + 25;
	else
		currentmoidlimit = self.MOIDLimit;
	end

	if MovableMan:GetMOIDCount() > currentmoidlimit then
		-- Disable cloning if we hit MOID limit
		self.CloningEnabled = false;
	else
		if self.CloningEnabled == false then
			-- Enable cloning if we below MOID limit
			self.CloningEnabled = true;
			-- Reset startup timer to start vats asynchronous mode
			self.CloningVatsDesyncTimer:Reset();
		end
	end

	for i = 1 , #self.Generators do
		if  MovableMan:IsParticle(self.Generators[i]) then
			if self.CloningEnabled == false then
				self.Generators[i]:EnableEmission(false);
			else
				-- Every vat starts with 2.5 sec delay
				if self.CloningVatsDesyncTimer:IsPastSimMS(i * 2500) then
					if not self.Generators[i]:IsEmitting() then
						self.Generators[i]:EnableEmission(true);
					end
				end
			end
		else
			self.Generators[i] = nil;
		end
	end
end
-----------------------------------------------------------------------------------------
-- Main "for actor in actors" loop to prevent additional actor enumerating loop and add delays
-----------------------------------------------------------------------------------------
function Swarm:ActorSearchLoop()
	-- Clear empty zones stats
	for i = 1 , #self.IsZoneEmpty do
		self.IsZoneEmpty[i] = true;
	end

	-- Clear taken zones stats
	for i = 1 , #self.IsBrainHallTaken do
		self.IsBrainHallTaken[i] = false;
	end

	-- Clear taken vats zones stats
	for i = 1 , #self.BaseVatsZonesTaken do
		self.BaseVatsZonesTaken[i] = false;
	end
	
	self.CoalitionCount = 0;
	self.DummyCount = 0;
	self.UnitCount = 0;

	self.BrainThreatened = false;
	self.VatsThreatened = false;

	-- OutpostZone Includes outpost firing points 1 to 5
	-- ScanZone1 Includes all firing point from 6 to 15
	-- ScanZone2 Includes all firing point from 16 to 23
	-- Loop actors and fill any stats
	for actor in MovableMan.Actors do
		-- Check for empty firing points
		if actor.Team == self.CPUTeam then
			if self.OutpostZone:IsInside(actor.Pos) then 
				for i = 1 , 5 do
					if self.Zones[i]:IsInside(actor.Pos) then 
						self.IsZoneEmpty[i] = false;
					end
				end
			elseif self.ScanZone1:IsInside(actor.Pos) then 
				for i = 6 , 15 do
					if self.Zones[i]:IsInside(actor.Pos) then 
						self.IsZoneEmpty[i] = false;
					end
				end
			elseif self.ScanZone2:IsInside(actor.Pos) then 
				for i = 16 , 23 do
					if self.Zones[i]:IsInside(actor.Pos) then 
						self.IsZoneEmpty[i] = false;
					end
				end
			end
		end
		
		-- This part can be optimized the same way as firing point but only if nessesary
		-- Check for taken brain hall areas
		if actor.Team == self.PlayerTeam then
			for i = 1 , #self.IsBrainHallTaken do
				if self.Zones[self.BaseBrainHallRight - 1 + i]:IsInside(actor.Pos) then 
					self.IsBrainHallTaken[i] = true;
					self.BrainThreatened = true;
				end
			end

			for i = 1 , #self.BaseVatsZonesTaken do
				if self.BaseVatsZones[i]:IsInside(actor.Pos) then 
					self.BaseVatsZonesTaken[i] = true;
					self.VatsThreatened = true;
				end
			end
		end
		
		-- Count actor stats
		if actor.Team == self.CPUTeam then
			self.CoalitionCount = self.CoalitionCount + 1;
		end

		if actor.Team == self.PlayerTeam then
			self.DummyCount = self.DummyCount + 1;
		end
		
		self.UnitCount = self.UnitCount + 1;
	end -- for
	
end
-----------------------------------------------------------------------------------------
-- Reutrns true if power generators were destroyed
-----------------------------------------------------------------------------------------
function Swarm:PowerGeneratorsDestroyed()
	local destroyed = true;
	
	for i = 1 , #self.PowerGenerators do 
		if MovableMan:IsParticle(self.PowerGenerators[i]) then
			destroyed = false;
			break;
		else
			self.PowerGenerators[i] = nil;
		end
	end

	return destroyed;
end
-----------------------------------------------------------------------------------------
-- Enumerates actors and sends them to appropriate destinations
-----------------------------------------------------------------------------------------
function Swarm:TeleportControlLoop()
	-- TeleportControlLoop should follow ActorSearchLoop to have decision-making stats 
	-- already available
	-- Check for actors inside teleporters
	for actor in MovableMan.Actors do
		-- First we check if an actor is inside vats chamber
		-- and if it is we check all A-teleport areas
		if self.BaseVatsChamber:IsInside(actor.Pos) then
			for i = 1 , #self.TeleportersA do
				if self.TeleportersA[i]:WithinBox(actor.Pos) then 
					self:SelectTeleportationDest(actor);
				end
			end
		end
	end -- for
end
-----------------------------------------------------------------------------------------
-- Selects B-teleport according to battlefield situation
-----------------------------------------------------------------------------------------
function Swarm:SelectTeleportationDest(actor)
	local teleportsuccess = false;
	local loopstart;
	-- Debug only
	--local selectedpoint = 0;
	--local selecteddest = 0;
	--local selectedpointdesc = "";
	
	if not teleportsuccess then
		-- First we send troops if brain hall is taken by enemy
		for i = 1 , #self.IsBrainHallTaken do
			local ii = self.BaseBrainHallRight - 1 + i;
			-- If enemy taken brain hall zone then send troops there discarding any other empty points
			if self.IsBrainHallTaken[i] == true then
				if self.NextTroopsSendTime[ii] < self.TeleportationTimer.ElapsedSimTimeMS then
					if self.NextTeleporterReceiveTime[self.ZoneToTeleport[ii]] < self.TeleportationTimer.ElapsedSimTimeMS then
						-- Debug
						--selectedpointdesc = "Teleport: Brain hall attack";
						--selectedpoint = ii;
						--selecteddest = self.ZoneToTeleport[ii];
						-- end debug

						self:Teleport(actor.Pos , self.TeleportersB[self.ZoneToTeleport[ii]] , actor);
						teleportsuccess = true;

						-- Adjust send\receive time values
						self.NextTroopsSendTime[ii] = self.TeleportationTimer.ElapsedSimTimeMS + self.FiringPointReachDelay / 4;
						-- Brain protecting B-ports will cool down after 2.5 secs
						self.NextTeleporterReceiveTime[self.ZoneToTeleport[ii]] = self.TeleportationTimer.ElapsedSimTimeMS  + self.TeleporterReceiveDelay / 4;

						actor:AddInventoryItem(CreateHDFirearm(self.ZoneWeapons[ii][math.random(#self.ZoneWeapons[ii])] , "Coalition.rte"));
						actor.AIMode = Actor.AIMODE_GOTO;
						actor:ClearAIWaypoints();
						actor:AddAISceneWaypoint(self.ZoneDestinations[ii]);
						break;
					end -- if
				end -- if
			end -- if
		end -- for
	end  -- if

	if not teleportsuccess then
		-- Then we send troops if cloning vats are taken by enemy
		for i = 1 , #self.BaseVatsZonesTaken do
			-- If enemy taken vats zone then send troops there discarding any other empty points
			if self.BaseVatsZonesTaken[i] == true then
				-- Manually select common zone depending on compromised vats zone
				if i == 1 then
					ii = self.BaseChamberUp;
				elseif i == 2 then
					ii = self.BaseRoof;
				else 
					ii = self.BaseChamberMid;
				end
			
				if self.NextTroopsSendTime[ii] < self.TeleportationTimer.ElapsedSimTimeMS then
					if self.NextTeleporterReceiveTime[self.ZoneToTeleport[ii]] < self.TeleportationTimer.ElapsedSimTimeMS then
						-- Debug
						--selectedpointdesc = "Teleport: Vats attack";
						--selectedpoint = ii;
						--selecteddest = self.ZoneToTeleport[ii];
						-- end debug

						self:Teleport(actor.Pos , self.TeleportersB[self.ZoneToTeleport[ii]] , actor);
						teleportsuccess = true;

						-- Adjust send\receive time values
						self.NextTroopsSendTime[ii] = self.TeleportationTimer.ElapsedSimTimeMS + self.FiringPointReachDelay / 4;
						-- Vats protecting B-ports will cool down after ~3.5 secs then ordinary,
						-- because vats are valuable and teleports too far from vats
						self.NextTeleporterReceiveTime[self.ZoneToTeleport[ii]] = self.TeleportationTimer.ElapsedSimTimeMS  + self.TeleporterReceiveDelay / 4;
						-- self.NextTeleporterReceiveTime[self.ZoneToTeleport[ii]] = self.TeleportationTimer.ElapsedSimTimeMS + self.TeleporterReceiveDelay;

						actor:AddInventoryItem(CreateHDFirearm(self.ZoneWeapons[ii][math.random(#self.ZoneWeapons[ii])] , "Coalition.rte"));
						actor.AIMode = Actor.AIMODE_GOTO;
						actor:ClearAIWaypoints();
						actor:AddAISceneWaypoint(self.ZoneDestinations[ii]);
						break;
					end -- if
				end -- if
			end -- if
		end -- for
	end  -- if
	
	if not teleportsuccess then
		-- Then we send troops if brain hall is empty
		for i = self.BaseBrainHallRight , self.BaseBrainHallLeft do
			if self.IsZoneEmpty[i] == true and self.NextTroopsSendTime[i] < self.TeleportationTimer.ElapsedSimTimeMS then
				if self.NextTeleporterReceiveTime[self.ZoneToTeleport[i]] < self.TeleportationTimer.ElapsedSimTimeMS then
					-- Debug
					--selectedpointdesc = "Teleport: Brain hall empty";
					--selectedpoint = i;
					--selecteddest = self.ZoneToTeleport[i];
					-- end debug

					self:Teleport(actor.Pos , self.TeleportersB[self.ZoneToTeleport[i]] , actor);
					teleportsuccess = true;
					
					self.NextTroopsSendTime[i] = self.TeleportationTimer.ElapsedSimTimeMS + self.FiringPointReachDelay;
					self.NextTeleporterReceiveTime[self.ZoneToTeleport[i]] = self.TeleportationTimer.ElapsedSimTimeMS + self.TeleporterReceiveDelay;
					
					actor:AddInventoryItem(CreateHDFirearm(self.ZoneWeapons[i][math.random(#self.ZoneWeapons[i])] , "Coalition.rte"));
					actor.AIMode = Actor.AIMODE_GOTO;
					actor:ClearAIWaypoints();
					actor:AddAISceneWaypoint(self.ZoneDestinations[i]);
					break;
				end -- if
			end -- if
		end -- for
	end  -- if

	if not teleportsuccess then
		-- Generic destination search
		if self:PowerGeneratorsDestroyed() or self.CurrentFightPhase == self.FightPhase.OUTPOSTCAPTURED then
			-- Skip 5 first zones if power generators were destroyed or initial stage complete
			loopstart = 6; 
		else
			loopstart = 1;
		end
		
		for i = loopstart , #self.IsZoneEmpty do
			if self.IsZoneEmpty[i] == true and self.NextTroopsSendTime[i] < self.TeleportationTimer.ElapsedSimTimeMS then
				if self.NextTeleporterReceiveTime[self.ZoneToTeleport[i]] < self.TeleportationTimer.ElapsedSimTimeMS then
					-- Debug
					--selectedpointdesc = "Teleport: Standard";
					--selectedpoint = i;
					--selecteddest = self.ZoneToTeleport[i];
					-- End debug

					self:Teleport(actor.Pos , self.TeleportersB[self.ZoneToTeleport[i]] , actor);
					teleportsuccess = true;

					self.NextTroopsSendTime[i] = self.TeleportationTimer.ElapsedSimTimeMS + self.FiringPointReachDelay;
					self.NextTeleporterReceiveTime[self.ZoneToTeleport[i]] = self.TeleportationTimer.ElapsedSimTimeMS + self.TeleporterReceiveDelay;

					actor:AddInventoryItem(CreateHDFirearm(self.ZoneWeapons[i][math.random(#self.ZoneWeapons[i])] , "Coalition.rte"));
					actor.AIMode = Actor.AIMODE_GOTO;
					actor:ClearAIWaypoints();
					actor:AddAISceneWaypoint(self.ZoneDestinations[i]);
					break;
				end -- if
			end -- if
		end -- for
	end  -- if
	
	-- Appropriate teleport destination was not selected, send clone to default destination
	-- and make it hunt for player brain
	if not teleportsuccess then
		selectedpointdesc = "Teleport: Default";
		selectedpoint = self.TeleporterBBaseBrainHallRight;

		self:Teleport(actor.Pos , self.TeleportersB[self.TeleporterBBaseBrainHallRight] , actor);
		actor.AIMode = Actor.AIMODE_BRAINHUNT;
		actor:AddInventoryItem(CreateHDFirearm("Assault Rifle" , "Coalition.rte"));
	end

	-- Debug
	print ("Troops:");
	print (string.format("Clones: %d - Dummy: %d - Ratio: %.3f - Total: %d" , self.CoalitionCount , self.DummyCount , self.CoalitionCount / self.DummyCount , self.UnitCount));
	
	--[[
	print (selectedpointdesc);
	print (selectedpoint);
	print (self.ZonesDesc[selectedpoint]);
	print (self.TeleportersBDesc[selecteddest]);]]--
end
-----------------------------------------------------------------------------------------
-- Teleports actor with effects(Vector)from , (Vector)to , (Actor)actor
-----------------------------------------------------------------------------------------
function Swarm:Teleport(from , to , actor)
	--Teleport the actor.
	actor.Pos = to;
	--Make the actor glow for a while.
	actor:FlashWhite(1500);
	--Create the teleportation effect for both teleporters in the set.
	local fxa = CreateAEmitter("Teleporter Effect A");
	fxa.Pos = from;
	MovableMan:AddParticle(fxa);
	local fxb = CreateAEmitter("Teleporter Effect B");
	fxb.Pos = to;
	MovableMan:AddParticle(fxb);
end
-----------------------------------------------------------------------------------------
-- Send dummy reinforcements
-----------------------------------------------------------------------------------------
function Swarm:SendReinforcements()
	if self.DummyCount / self.CoalitionCount > 0.65 then
		self.DummyDropshipSpawnTimer:Reset();
		self.DummySpawnTimer:Reset();
		self.DummyGoldTimer:Reset();
	end

	-- Give some gold
	--[[if self.DummyGoldTimer:IsPastSimMS(self.DummyGoldInterval) and self:GetTeamFunds(self.PlayerTeam) < 800 then
		self:SetTeamFunds(self:GetTeamFunds(self.PlayerTeam) + self.DummyGoldAmount , self.PlayerTeam);
		self.DummyGoldTimer:Reset();
	end]]--
	
	-- Remove all player gold as it added when ships get orbited
	self:SetTeamFunds(0 , self.PlayerTeam);
	
	-- Spawn ships
	if self.DummyDropshipSpawnTimer:IsPastSimMS(self.DummyDropshipSpawnInterval) and MovableMan:GetMOIDCount() <= self.MOIDLimit then
		print("Spawn ship");				
		if math.random() < 0.95 then
			-- Add dummies and ship
			local ship = CreateACDropShip("Drop Ship","Dummy.rte");
			for i = 0 , 1 do
				print("Spawn dummies");				
				local actor = self:CreateDummy();
				actor.Team = self.PlayerTeam;
				ship:AddInventoryItem(actor);
			end

			-- Select LZ according to current fight stage
			if self.CurrentFightPhase == self.FightPhase.NOFIGHT  then
				ship.Pos = Vector(self.StartLZ:GetRandomPoint().X, 0);
			elseif self.CurrentFightPhase == self.FightPhase.OUTPOSTTAKEN  then
				ship.Pos = Vector(self.OutpostBridge:GetRandomPoint().X, 0);
			end
			
			ship.Team = self.PlayerTeam;
			ship:SetControllerMode(Controller.CIM_AI, -1);
			MovableMan:AddActor(ship);
		else
			local ship = CreateACDropShip("Drop Ship","Dummy.rte");
			local actor = CreateACrab("Dreadnought");
			actor.Team = self.PlayerTeam;
			ship:AddInventoryItem(actor);

			-- Select LZ according to current fight stage
			if self.CurrentFightPhase == self.FightPhase.NOFIGHT  then
				ship.Pos = Vector(self.OutpostBridge:GetRandomPoint().X, 0);
			elseif self.CurrentFightPhase == self.FightPhase.OUTPOSTTAKEN  then
				ship.Pos = Vector(self.BaseBridge:GetRandomPoint().X, 0);
			end
			
			ship.Team = self.PlayerTeam;
			ship:SetControllerMode(Controller.CIM_AI, -1);
			MovableMan:AddActor(ship);
		end
		self.DummyDropshipSpawnTimer:Reset();
		print("Ship spawn complete");	
	end
	
	-- Spawn dummies
	if self.DummySpawnTimer:IsPastSimMS(self.DummySpawnInterval) and MovableMan:GetMOIDCount() <= self.MOIDLimit then
		local actor = self:CreateDummy();
		actor.Team = self.PlayerTeam;
		actor.Pos = Vector(4595 , 1050);
		MovableMan:AddActor(actor);
		self.DummySpawnTimer:Reset();
	end
end
-----------------------------------------------------------------------------------------
-- Create dummy trooper, returns created actor
-----------------------------------------------------------------------------------------
function Swarm:CreateDummy()
	local actor = nil;
	actor = CreateAHuman("Dummy");

	if self.CurrentFightPhase == self.FightPhase.NOFIGHT then
		actor:AddInventoryItem(CreateHDFirearm(self.LightWList[math.random(#self.LightWList)],"Dummy.rte"));
		-- actor:AddInventoryItem(CreateHDFirearm("Blaster","Dummy.rte"));
	elseif self.CurrentFightPhase == self.FightPhase.OUTPOSTTAKEN then
		actor:AddInventoryItem(CreateHDFirearm(self.HeavyWList[math.random(#self.HeavyWList)],"Dummy.rte"));
		-- actor:AddInventoryItem(CreateHDFirearm(self.LightWList[math.random(#self.LightWList)],"Dummy.rte"));
	end

	if MovableMan:IsActor(self.CPUBrain) then
		actor.AIMode = Actor.AIMODE_GOTO;
		actor:ClearAIWaypoints();
		actor:AddAISceneWaypoint(self.CPUBrain.Pos);
	end
	
	return actor;
end
-----------------------------------------------------------------------------------------
-- Phase NOFIGHT processing
-----------------------------------------------------------------------------------------
function Swarm:PhaseNoFight()
	--Mark commander on start for 10 seconds to warn player
	if not self.GenericTimer:IsPastSimMS(15000) then
		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				if self.AssignedBrains[player]:HasObjectInGroup("Brains") then
					self:AddObjectivePoint("Protect commander!", self.AssignedBrains[player].AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
				end
			end -- if
		end -- for

		self:AddObjectivePoint("CAPTURE OUTPOST", Vector(2460 , 500) , self.PlayerTeam, GameActivity.ARROWDOWN);
		
		for player = 0, self.PlayerCount - 1 do
			self:ResetMessageTimer(player);
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("CAPTURE COALITION OUTPOST THEN ADVANCE TO BASE", player, 0, 5000, true);
		end	
	end -- if

	local liveenemies = 0;
	for actor in MovableMan.Actors do
		if actor.ClassName ~= "ADoor" and actor.Team == self.CPUTeam and self.OutpostZone:IsInside(actor.Pos) then
			liveenemies = liveenemies + 1;
		end
	end -- for

	for actor in MovableMan.AddedActors do
		if actor.ClassName ~= "ADoor" and actor.Team == self.CPUTeam and self.OutpostZone:IsInside(actor.Pos) then
			liveenemies = liveenemies + 1;
		end
	end -- for
	
	if not self:PowerGeneratorsDestroyed() then
		self:AddObjectivePoint("DESTROY GENERATORS TO DISABLE TELEPORTS", Vector(2294, 865) , self.PlayerTeam, GameActivity.ARROWDOWN);
	end

	if liveenemies == 0 and self:PowerGeneratorsDestroyed() then
		self:AdvanceToOutpostTaken();
	end -- if
end
-----------------------------------------------------------------------------------------
-- Phase NOFIGHT -> OUTPOSTTAKEN processing
-----------------------------------------------------------------------------------------
function Swarm:AdvanceToOutpostTaken()
	print("Outpost captured");

	for player = 0, self.PlayerCount - 1 do
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		FrameMan:SetScreenText("OUTPOST CAPTURED ADVANCE TO BASE", player, 0, 10000, true);
	end

	self.DummyDropshipSpawnInterval = 12000;
	self.DummySpawnInterval = 15500;

	self.GenericTimer:Reset();
	self.CurrentFightPhase = self.FightPhase.OUTPOSTTAKEN;
end
-----------------------------------------------------------------------------------------
-- Phase OUTPOSTTAKEN processing
-----------------------------------------------------------------------------------------
function Swarm:PhaseOutpostTaken()

end
-----------------------------------------------------------------------------------------
-- Check if we are victorious
-----------------------------------------------------------------------------------------
function Swarm:CheckWinningConditions()
	if self.CPUBrainDead == false then
		if not MovableMan:IsActor(self.CPUBrain)then
			-- Team 1 wins!
			self.CPUBrainDead = true;
			self.WinnerTeam = self.PlayerTeam;
			ActivityMan:EndActivity();
			-- Tell the script to stop running for brains as the game has ended.
			for player = 0, self.PlayerCount - 1 do
				self.braindead[player] = true;
			end
		else
			self:AddObjectivePoint("KILL", self.CPUBrain.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------

































